/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            camera.h
 *
 *  Fri Oct 29 12:46:38 CEST 2004
 *  Copyright  2004 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id: camera.h,v 1.10 2005/05/03 08:31:59 deva Exp $
 */

/*
 * $Log: camera.h,v $
 * Revision 1.10  2005/05/03 08:31:59  deva
 * Removed the error object, and replaced it with a more generic info object.
 *
 * Revision 1.9  2005/05/01 11:25:56  deva
 * Added code to read screenshot from frame queue, decode it to rgb and put it into a qimage.
 *
 * Revision 1.8  2005/05/01 09:56:26  deva
 * Added Id and Log tags to all files
 */

#include <config.h>
#ifdef USE_GUI

#ifndef __CAMERA_H__
#define __CAMERA_H__

#include <string>
using namespace std;
#include "info.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <avformat.h>

#include "util.h"
#include "queue.h"
#include "decoder.h"
#include "encoder.h"
#include "player.h"
#include "package.h"

#include "thread.h"
#include "frame.h"

#include <qwidget.h>

/**
 * This class represents the symbolic representation of the camera and 
 * the network functionality.
 */
class Camera {
public:
  Camera(Info *ginfo);
  ~Camera();
  void connect(const char *ip,
               const int port);

  void setCpr(char *newcpr);

  // Camera actions
  void start();
  void stop(n_savestate save);
  void freeze();
  void unfreeze();
  void snapshot(unsigned char *rgb);

private:
  // Info object passed to all sub objects.
  Info *info;
  bool initialized;

	pthread_t playertid;
	pthread_t decodetid;
	pthread_t encodetid;
	volatile int running;

  Encoder *encoder;
  Decoder *decoder;
  Player *player;

  Queue<Frame> *encode_queue;
  Queue<Frame> *player_queue;
  sem_t encode_sem;
  sem_t player_sem;
  pthread_mutex_t mutex;// = PTHREAD_MUTEX_INITIALIZER;
};


#endif/*__CAMERA_H__*/

#endif/* USE_GUI */
