/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            decoder.h
 *
 *  Sat Feb 19 17:05:42 CET 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 * Originally from:
 * RTVideoRec Realtime video recoder and encoder for Linux
 *
 * Copyright (C) 2004  B. Stultiens
 */

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id: decoder.h,v 1.12 2005/05/03 08:31:59 deva Exp $
 */

/*
 * $Log: decoder.h,v $
 * Revision 1.12  2005/05/03 08:31:59  deva
 * Removed the error object, and replaced it with a more generic info object.
 *
 * Revision 1.11  2005/05/02 14:06:55  deva
 * Added unfreeze to decoder (called from camera).
 *
 * Revision 1.10  2005/05/02 10:35:23  deva
 * Fixed wrongly showed snapshot thumbnails.
 *
 * Revision 1.9  2005/05/02 09:50:22  deva
 * Rewrote freeze, shoot and record flags, from encoder to frame.
 *
 * Revision 1.8  2005/05/01 09:56:26  deva
 * Added Id and Log tags to all files
 */

#include <config.h>
#ifdef USE_GUI

#ifndef __RTVIDEOREC_DECODER_H
#define __RTVIDEOREC_DECODER_H

#include "info.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "queue.h"
#include "encoder.h"
#include "player.h"

#include "thread.h"
#include "frame.h"

class Decoder : public Thread {
public:
  Decoder(Info *ginfo,
          sem_t *gencode_sem,
          sem_t *gplayer_sem,
          Queue<Frame> *gencode_queue,
          Queue<Frame> *gplayer_queue,
          pthread_mutex_t *gmutex,
          volatile int *grunning);
  ~Decoder();
  void run();

  void freeze();
  void unfreeze();
  void shoot(unsigned char *rgb);
  void start();
  void stop(n_savestate save);

private:
  void getScreenshot(Frame *frame, unsigned char *rgb);

  pthread_mutex_t shot_mutex;
  Frame* shot;

  volatile bool b_freeze;
  volatile bool b_shoot;
  volatile bool b_record;

  Info *info;
  AVCodecContext dvcodec;

  sem_t *encode_sem;
  sem_t *player_sem;
  Queue<Frame> *encode_queue;
  Queue<Frame> *player_queue;
  pthread_mutex_t *mutex;
  volatile int *running;

  void decode();
};

#endif/* __RTVIDEOREC_DECODER_H*/

#endif/*USE_GUI*/
