/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            encoder.cc
 *
 *  Tue Apr 19 12:10:34 CEST 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 * Originally from:
 * RTVideoRec Realtime video recoder and encoder for Linux
 *
 * Copyright (C) 2004  B. Stultiens
 * Copyright (C) 2004  Koen Otter and Glenn van der Meyden
 */

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id: encoder.cc,v 1.26 2005/05/03 08:31:59 deva Exp $
 */

/*
 * $Log: encoder.cc,v $
 * Revision 1.26  2005/05/03 08:31:59  deva
 * Removed the error object, and replaced it with a more generic info object.
 *
 * Revision 1.25  2005/05/02 20:34:38  deva
 * Some hacked borky ugly scumm code to check for errors! :(
 *
 * Revision 1.24  2005/05/02 10:35:23  deva
 * Fixed wrongly showed snapshot thumbnails.
 *
 * Revision 1.23  2005/05/02 10:18:51  deva
 * Preserve network connection when a frozen frame exists, even though no recording is done.
 *
 * Revision 1.22  2005/05/02 10:01:58  deva
 * Create and destroy netowrk connections in main function and not in start/stop/freeze... etc
 *
 * Revision 1.21  2005/05/02 09:50:22  deva
 * Rewrote freeze, shoot and record flags, from encoder to frame.
 *
 * Revision 1.20  2005/05/02 09:18:13  deva
 *
 * Fixed decoding problem in snapshot thumbnails.
 *
 * Revision 1.19  2005/05/01 12:23:20  deva
 *
 * Now snapshot is read from frame queue.
 *
 * Revision 1.18  2005/05/01 11:25:56  deva
 * Added code to read screenshot from frame queue, decode it to rgb and put it into a qimage.
 *
 * Revision 1.17  2005/05/01 09:56:26  deva
 * Added Id and Log tags to all files
 */

#include <config.h>
#ifdef USE_GUI

#include "encoder.h"

Encoder::Encoder(Info *ginfo,
                 const char *gip,
                 const int gport,
                 sem_t	*gsem,
                 Queue<Frame> *gqueue,
                 pthread_mutex_t *gmutex,
                 volatile int *grunning)
{
  info = ginfo;

  strcpy(ip, gip);
  port = gport;
  memset(cpr, 0, sizeof(cpr));

  sem = gsem;
  queue = gqueue;
  mutex = gmutex;
  running = grunning;

  //  record = 0;

 	sem_init(&record_sem, 0, 0);

  s = NULL;
  n = NULL;

  frozen = false;

  //  shoot_request = 0;
  //  shoot_value = 0;
  //  freeze_request = 0;
  //  freeze_value = 0;
}


Encoder::~Encoder()
{
}


void Encoder::encode()
{ 
  Frame *frame;

  while(*running) {
    sem_wait(sem);

    frame = queue->pop();
    
    if(frame) {
      if(frame->freeze) frozen = true;
      if(frame->shoot) frozen = false;

      if(frame->record || 
         frame->freeze || 
         frame->shoot) {

        // If no connection is present, make a new one
        if(!s) {
          s = new Socket(port, info);
          s->sconnect(ip);
          n = new Network(s, info);
        }

        n_header h;
      
        h.header_type = DATA_HEADER;
        sprintf(h.header.h_data.cpr, cpr);
        h.header.h_data.freeze = frame->freeze;
        h.header.h_data.snapshot = frame->shoot;
        h.header.h_data.record = frame->record;
        h.header.h_data.savestate = NO_CHANGE;
        
        //        if(freeze_request != freeze_value) freeze_value = freeze_request;
        //        if(shoot_request != shoot_value) shoot_value = shoot_request;
        
        n->sendPackage(&h, frame->data, frame->size);
      } else {
        // When frozen we need to preserve the connection in order to
        // remember the frozen frame on the server side.
        if(!frozen) {
          // No data is to be sent, if we have a connection, destroy it.
          if(s) {
            if(n) delete n;
            delete s;
            s = NULL;
            n = NULL;
          }
        }
      }
      if(frame) delete frame;
    }
  }
}


void Encoder::setCpr(char *newcpr)
{
  strcpy(cpr, newcpr);
}


void Encoder::freeze()
{ 
  /* 
  if(!s) {
    s = new Socket(port, errobj);
    s->sconnect(ip);
    n = new Network(s, errobj);
  }
  */
  //  if(!errobj->hasError()) freeze_request = 1 - freeze_request;
}


/*
 * shoot
 * Set the shoot bit in the network header on the current frame.
 * return the decodet (rgba) version af that frame, for thumbnail show.
 */
void Encoder::shoot()
{
  /*
  if(!s) {
    s = new Socket(port, errobj);
    s->sconnect(ip);
    n = new Network(s, errobj);
  }
  */
  //  if(!errobj->hasError()) shoot_request = 1 - shoot_request;
  //  getScreenshot(rgb);
}


void Encoder::run() {
  encode();
  if(s) {
    if(n) delete n;
    delete s;
    s = NULL;
    n = NULL;
  }
  fprintf(stderr, "Encoder thread stopped.\n"); fflush(stderr);
}


void Encoder::start() {
  /*
  if(!s) {
    s = new Socket(port, errobj);
    s->sconnect(ip);
    n = new Network(s, errobj);
  }
  */
  //  if(!errobj->hasError()) record = 1;
}


void Encoder::stop(n_savestate save) {
/*
  struct timespec ts;
  // TODO: set save state in package header.

  // Lock the queue and wait until all elements are sent on the network.
  queue->lock();
  fprintf(stderr, "Emptying queue"); fflush(stderr);
  while(queue->peek()) {
    // Remove any late buffer
    // We don't care, the encoder finishes them all
    ts.tv_sec = 0;
    ts.tv_nsec = 500000000L;	// 100ms
    fprintf(stderr, "."); fflush(stderr);
    nanosleep(&ts, NULL);
  }
  fprintf(stderr, "done!\n"); fflush(stderr);

  record = 0;

  queue->unlock();
*/
/*
  if(s) {
    if(n) delete n;
    delete s;
    s = NULL;
    n = NULL;
  }
*/
}

#endif /*USE_GUI*/
