/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            miav.cc
 *
 *  Sat Aug 21 17:32:24 2004
 *  Copyright  2004  deva
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id: miav.cc,v 1.8 2005/05/03 09:22:12 deva Exp $
 */

/*
 * $Log: miav.cc,v $
 * Revision 1.8  2005/05/03 09:22:12  deva
 * Implemented the gui part of the info object.
 *
 * Revision 1.7  2005/05/03 08:31:59  deva
 * Removed the error object, and replaced it with a more generic info object.
 *
 * Revision 1.6  2005/05/02 19:47:43  deva
 * Fixed overlapping cpr numbers on the server (now it saves one cpr pr.
 * connection, and ignores any changes sent)
 *
 * Revision 1.5  2005/05/01 09:56:26  deva
 * Added Id and Log tags to all files
 */

#include <config.h>

#include "server.h"
#include "socket.h"

#ifdef USE_GUI
#include <qapplication.h>
#include "mainwindow.h"
#include "miav.h"
#endif /* USE_GUI */

#include "miav_config.h"

#include "info_console.h"
#include "info_gui.h"

#include <stdio.h>

enum {
  MODE_GRAB,
  MODE_SERVER
};


/** 
 * This function starts the MIaV gui.
 */
int grab(int argc, char *argv[]) {
#ifdef USE_GUI
  QApplication miav_grab( argc, argv );

  InfoGui info(&miav_grab, NULL);
  config = new MiavConfig(ETC"/miav.conf", &info);

  MainWindow mainwindow(&miav_grab);
  miav_grab.setMainWidget( &mainwindow );
  return miav_grab.exec();
#else /* USE_GUI */
  printf("Error: MIaV was not compiled with GUI support...\n");
  return 0;
#endif /* USE_GUI */
}


/** 
 * This function starts the MIaV server.
 */
int server(int argc, char *argv[]) {
  int port = config->readInt("server_port");
  pid_t childpid; // variable to store the child's pid

  signal(SIGCLD, SIG_IGN);  // Ved SIGCHILD til IGNORE maa wait/waitpid ikke kaldes 
                            //   (ellers kommer der kernel-brok)
  
  printf("Listening on port %d\n", port);
  Socket *socket = new Socket(port);

  while(1) {
    Socket *csocket = new Socket(socket->slisten());
    if(csocket->isConnected()) {
      childpid = fork();
      
      if(childpid == -1) {
        // fork() returns -1 on failure
        perror("fork");
        exit(1); 
      } else if(childpid == 0) {
        // fork() returns 0 to the child process
        delete socket; // Close listen socket.
        newConnection(csocket);
        delete csocket; // Close communication socket.
        exit(0);
      } else {
        // fork() returns new pid to the parent process
      }
    }
  }
  delete socket;
  return 0;
}

#include "debug.h"

int main(int argc, char *argv[])
{

  int mode = MODE_GRAB;

  if(argc < 2) {
    printf("Usage: miav [mode] [mode options]\n");
    printf("[mode] can be one of the following: grab or server.\n");
    return 1;
  }

  if(!strcmp(argv[1], "grab")) mode = MODE_GRAB;
  if(!strcmp(argv[1], "server")) mode = MODE_SERVER;

  switch(mode) {
  case MODE_GRAB:
    {
      return grab(argc - 2, &argv[2]);
    }
  case MODE_SERVER:
    {
      InfoConsole info;
      config = new MiavConfig(ETC"/miav.conf", &info);
      return server(argc - 2, &argv[2]);
    }
  }
  return 0;
}
